@@ -1,6 +1,6 @@
 """
 Author: Travis Hammond
-Version: 1_7_2020
+Version: 5_7_2020
 """
 
 
@@ -12,22 +12,22 @@ from tensorflow.keras.models import model_from_json
 
 try:
     from paiutils.neural_network import (
-        Trainner, Predictor, dense, conv2d, conv1d
+        Trainer, Predictor, dense, conv2d, conv1d
     )
     from paiutils.util_funcs import (
         load_directory_dataset, load_h5py
     )
 except ImportError:
     from neural_network import (
-        Trainner, Predictor, dense, conv2d, conv1d
+        Trainer, Predictor, dense, conv2d, conv1d
     )
     from util_funcs import (
         load_directory_dataset, load_h5py
     )
 
 
-class AutoencoderTrainner(Trainner):
-    """AutoencoderTrainner is used for loading, saving,
+class AutoencoderTrainer(Trainer):
+    """AutoencoderTrainer is used for loading, saving,
        and training keras autoencoder models.
     """
 
@@ -191,6 +191,183 @@ class AutoencoderPredictor(Predictor):
             super().__init__(path)
 
 
+class AutoencoderExtraDecoderTrainer(Trainer):
+    """Autoencoder with an Extra Decoder Trainer is an Autoencoder Trainer
+       with a extra decoder that can be trained to y-data.
+    """
+
+    def __init__(self, model, encoder_model, decoder_model,
+                 extra_decoder_model, data, file_loader_x=None,
+                 file_loader_y=None, include_y_data=True):
+        """Initializes train, validation, and test data.
+        params:
+            model: A compiled full keras model
+            encoder_model: The encoder part of the full model
+            decoder_model: The decoder part of the full model
+            extra_decoder_model: The extra decoder is trained
+                                 to map the encoder to a
+                                 different output
+                                 (not part of the full model)
+            data: A dictionary or string (path) containg train data,
+                  and optionally validation and test data.
+                  Ex. {'train_x': [...], 'validation_x: [...]}
+            file_loader_x: A function for loading each x data file
+            file_loader_y: A function for loading each y data file
+            include_y_data: A boolean, which determines if y-data should
+                            be appened with the x-data for training the
+                            autoencoder
+        """
+        assert isinstance(data, (str, dict)), (
+            'data must be either in a dictionary or a file/folder path'
+        )
+        self.model = model
+        self.encoder_model = encoder_model
+        self.decoder_model = decoder_model
+        self.extra_decoder_model = extra_decoder_model
+        self.train_data = None
+        self.validation_data = None
+        self.test_data = None
+        self.train_data2 = None
+        self.validation_data2 = None
+        self.test_data2 = None
+
+        if isinstance(data, str):
+            if os.path.isdir(data):
+                assert file_loader_x is not None
+                assert file_loader_y is not None
+                data = load_directory_dataset(data, file_loader_x,
+                                              file_loader_y=file_loader_y)
+            else:
+                assert data.split('.')[1] == 'h5'
+                data = load_h5py(data)
+        if isinstance(data, dict):
+            if 'train_x' in data and 'train_y' in data:
+                if include_y_data:
+                    self.train_data = np.vstack([data['train_x'],
+                                                 data['train_y']])
+                else:
+                    self.train_data = data['train_x']
+                self.train_data = (self.train_data, self.train_data)
+                self.train_data2 = [data['train_x'], data['train_y']]
+            else:
+                raise Exception('There must be a train dataset')
+            if 'validation_x' in data and 'validation_y' in data:
+                if include_y_data:
+                    self.validation_data = np.vstack([data['validation_x'],
+                                                    data['validation_y']])
+                else:
+                    self.validation_data = data['validation_x']
+                self.validation_data = (self.validation_data,
+                                        self.validation_data)
+                self.validation_data2 = [data['validation_x'],
+                                         data['validation_y']]
+            if 'test_x' in data and 'test_y' in data:
+                if include_y_data:
+                    self.test_data = np.vstack([data['test_x'],
+                                                data['test_y']])
+                else:
+                    self.test_data = data['test_x']
+                self.test_data = (self.test_data, self.test_data)
+                self.test_data2 = [data['test_x'], data['test_y']]
+        else:
+            raise ValueError('Invalid data')
+
+    def train_extra_decoder(self, epochs, batch_size=None,
+                            callbacks=None, verbose=True):
+        """Trains the extra decoder keras model on the outputs
+           of the assumingly trained encoder.
+        params:
+            epochs: An integer, which is the number of complete
+                    iterations to train
+            batch_size: An integer, which is the number of samples
+                        per graident update
+            callbacks: A list of keras Callback instances,
+                       which are called during training and validation
+            verbose: A boolean, which determines the verbositiy level
+        """
+        train_data2_x = self.encoder_model.predict(
+            self.train_data2[0], batch_size=batch_size
+        )
+        validation_data2 = None
+        if self.validation_data2 is not None:
+            validation_data2_x = self.encoder_model.predict(
+                self.validation_data2[0], batch_size=batch_size
+            )
+            validation_data2 = (validation_data2_x,
+                                self.validation_data2[1])
+        self.extra_decoder_model.fit(train_data2_x, self.train_data2[1],
+                                     validation_data=validation_data2,
+                                     batch_size=batch_size, epochs=epochs,
+                                     verbose=1 if verbose else 0,
+                                     callbacks=callbacks)
+        if verbose:
+            print('Extra Decoder Train Data Evaluation: ', end='')
+            print(self.extra_decoder_model.evaluate(train_data2_x,
+                                                self.train_data2[1],
+                                                batch_size=batch_size,
+                                                verbose=0))
+            if self.validation_data2 is not None:
+                print('Extra Decoder Validation Data Evaluation: ', end='')
+                print(self.extra_decoder_model.evaluate(validation_data2[0],
+                                                        validation_data2[1],
+                                                        batch_size=batch_size,
+                                                        verbose=0))
+            if self.test_data2 is not None:
+                test_data2_x = self.encoder_model.predict(
+                    self.test_data2[0], batch_size=batch_size
+                )
+                print('Extra Decoder Test Data Evaluation: ', end='')
+                print(self.extra_decoder_model.evaluate(test_data2_x,
+                                                        self.test_data2[1],
+                                                        batch_size=batch_size,
+                                                        verbose=0))
+
+
+    def load(self, path, optimizer, loss, metrics=None):
+        """Loads a model and weights from a file.
+           (overrides the inital provided model)
+        params:
+            path: A string, which is the path to a folder
+                  containing model.json, weights.h5, note.txt
+                  and maybe encoder/decoder parts
+            optimizer: A string or optimizer instance, which will be
+                       the optimizer for the loaded model
+            loss: A string or loss instance, which will be
+                  the loss function for the loaded model
+            metrics: A list of metrics, which will be used
+                     by the loaded model
+        """
+        super().load(path, optimizer, loss, metrics=metrics)
+        if 'extra_decoder_model.json' in os.listdir(path):
+            edm_path = os.path.join(path, 'extra_decoder_model.json')
+            with open(edm_path, 'r') as file:
+                self.extra_decoder_model = model_from_json(file.read())
+                self.extra_decoder_model.compile(optimizer=optimizer,
+                                                 loss=loss,
+                                                 metrics=metrics)
+            self.extra_decoder_model.load_weights(
+                os.path.join(path, 'extra_decoder_weights.h5')
+            )
+
+    def save(self, path, note=None):
+        """Saves the model and weights to a file.
+        params:
+            path: A string, which is the path to create a folder in
+                  containing model.json, weights.h5, note.txt, and
+                  maybe encoder/decoder parts
+            note: A string, which is a note to save in the folder
+        return: A string, which is the given path + folder name
+        """
+        path = super().save(path, note=note)
+        self.extra_decoder_model.save_weights(
+            os.path.join(path, 'extra_decoder_weights.h5')
+        )
+        edm_path = os.path.join(path, 'extra_decoder_model.json')
+        with open(edm_path, 'w') as file:
+            file.write(self.extra_decoder_model.to_json())
+        return path
+
+
 def create_basic_dense_model(input_shape, units_list,
                              activation='relu',
                              output_activation='sigmoid',
@@ -416,10 +593,10 @@ if __name__ == '__main__':
                       loss='mse')
         model.summary()
 
-        trainner = AutoencoderTrainner(model, dataset, encoder_model=encoder,
-                                       decoder_model=decoder)
-        path = trainner.train(20, batch_size=32)
-        path = trainner.save('')
+        trainer = AutoencoderTrainer(model, dataset, encoder_model=encoder,
+                                     decoder_model=decoder)
+        path = trainer.train(20, batch_size=32)
+        path = trainer.save('')
 
     predictor = AutoencoderPredictor(
         path, uses_encoder_model=uses_encoder_model,
